@@ -545,14 +545,22 @@ static void set_invalid_key_error(struct connman_network *network)
 					CONNMAN_SERVICE_ERROR_INVALID_KEY);
 }
 
+static void set_connect_error(struct connman_network *network)
+{
+	struct connman_service *service;
+
+	service = __connman_service_lookup_from_network(network);
+
+	__connman_service_indicate_error(service,
+					CONNMAN_SERVICE_ERROR_CONNECT_FAILED);
+}
+
 void connman_network_set_ipv4_method(struct connman_network *network,
 					enum connman_ipconfig_method method)
 {
 	struct connman_service *service;
 	struct connman_ipconfig *ipconfig;
 
-	network->element.ipv4.method = method;
-
 	service = __connman_service_lookup_from_network(network);
 	if (service == NULL)
 		return;
@@ -570,8 +578,6 @@ void connman_network_set_ipv6_method(struct connman_network *network,
 	struct connman_service *service;
 	struct connman_ipconfig *ipconfig;
 
-	network->element.ipv6.method = method;
-
 	service = __connman_service_lookup_from_network(network);
 	if (service == NULL)
 		return;
@@ -602,6 +608,9 @@ void connman_network_set_error(struct connman_network *network,
 	case CONNMAN_NETWORK_ERROR_INVALID_KEY:
 		set_invalid_key_error(network);
 		break;
+	case CONNMAN_NETWORK_ERROR_CONNECT_FAIL:
+		set_connect_error(network);
+		break;
 	}
 }
 
@@ -662,8 +671,7 @@ static void dhcp_success(struct connman_network *network)
 	if (err < 0)
 		goto err;
 
-	__connman_service_indicate_state(service, CONNMAN_SERVICE_STATE_READY,
-						CONNMAN_IPCONFIG_TYPE_IPV4);
+	__connman_service_set_ipconfig_ready(service, CONNMAN_IPCONFIG_TYPE_IPV4);
 
 	return;
 
@@ -675,15 +683,11 @@ err:
 static void dhcp_failure(struct connman_network *network)
 {
 	struct connman_service *service;
-	struct connman_ipconfig *ipconfig_ipv4;
 
 	service = __connman_service_lookup_from_network(network);
 	if (service == NULL)
 		return;
 
-	ipconfig_ipv4 = __connman_service_get_ip4config(service);
-	__connman_ipconfig_address_remove(ipconfig_ipv4);
-
 	__connman_service_indicate_state(service, CONNMAN_SERVICE_STATE_IDLE,
 						CONNMAN_IPCONFIG_TYPE_IPV4);
 }
@@ -725,9 +729,8 @@ static int set_connected_fixed(struct connman_network *network)
 	if (err < 0)
 		goto err;
 
-	__connman_service_indicate_state(service,
-					CONNMAN_SERVICE_STATE_READY,
-					CONNMAN_IPCONFIG_TYPE_IPV4);
+	__connman_service_set_ipconfig_ready(service, CONNMAN_IPCONFIG_TYPE_IPV4);
+
 	return 0;
 
 err:
@@ -763,8 +766,7 @@ static void set_connected_manual(struct connman_network *network)
 
 	connman_network_set_associating(network, FALSE);
 
-	__connman_service_indicate_state(service, CONNMAN_SERVICE_STATE_READY,
-					CONNMAN_IPCONFIG_TYPE_IPV4);
+	__connman_service_set_ipconfig_ready(service, CONNMAN_IPCONFIG_TYPE_IPV4);
 
 	return;
 
@@ -819,31 +821,22 @@ static int manual_ipv6_set(struct connman_network *network,
 static void autoconf_ipv6_set(struct connman_network *network)
 {
 	struct connman_service *service;
-	struct connman_ipconfig *ipconfig;
-	const char *nameserver = NULL;
 
 	DBG("network %p", network);
 
 	service = __connman_service_lookup_from_network(network);
 
-	ipconfig = __connman_service_get_ip6config(service);
-
 	__connman_device_increase_connections(network->device);
 
 	__connman_device_set_network(network->device, network);
 
 	connman_device_set_disconnected(network->device, FALSE);
 
-	connman_element_get_value(&network->element,
-			CONNMAN_PROPERTY_ID_IPV6_NAMESERVER, &nameserver);
-	if (nameserver != NULL)
-		__connman_service_nameserver_append(service, nameserver);
+	/* XXX: Append IPv6 nameservers here */
 
 	network->connecting = FALSE;
 
-	__connman_service_indicate_state(service,
-					CONNMAN_SERVICE_STATE_READY,
-					CONNMAN_IPCONFIG_TYPE_IPV6);
+	__connman_service_set_ipconfig_ready(service, CONNMAN_IPCONFIG_TYPE_IPV6);
 }
 
 static gboolean set_connected(gpointer user_data)
@@ -963,8 +956,8 @@ int connman_network_set_connected(struct connman_network *network,
 
 	if ((network->connecting == TRUE || network->associating == TRUE) &&
 							connected == FALSE) {
-		connman_element_set_error(&network->element,
-					CONNMAN_ELEMENT_ERROR_CONNECT_FAILED);
+		connman_network_set_error(network,
+					CONNMAN_NETWORK_ERROR_CONNECT_FAIL);
 		__connman_network_disconnect(network);
 	}
 
@@ -1011,7 +1004,6 @@ connman_bool_t connman_network_get_associating(struct connman_network *network)
  */
 int __connman_network_connect(struct connman_network *network)
 {
-	struct connman_service *service;
 	int err;
 
 	DBG("network %p", network);
@@ -1035,8 +1027,6 @@ int __connman_network_connect(struct connman_network *network)
 
 	__connman_device_disconnect(network->device);
 
-	service = __connman_service_lookup_from_network(network);
-
 	err = network->driver->connect(network);
 	if (err < 0) {
 		if (err == -EINPROGRESS)
@@ -1107,8 +1097,7 @@ static int manual_ipv4_set(struct connman_network *network,
 
 	__connman_ipconfig_gateway_add(ipconfig);
 
-	__connman_service_indicate_state(service, CONNMAN_SERVICE_STATE_READY,
-					CONNMAN_IPCONFIG_TYPE_IPV4);
+	__connman_service_set_ipconfig_ready(service, CONNMAN_IPCONFIG_TYPE_IPV4);
 
 	return 0;
 }
@@ -1176,7 +1165,7 @@ int __connman_network_set_ipconfig(struct connman_network *network,
 			if (ret != 0) {
 				connman_network_set_error(network,
 					CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL);
-				return FALSE;
+				return ret;
 			}
 			break;
 		case CONNMAN_IPCONFIG_METHOD_DHCP:
@@ -1785,9 +1774,6 @@ static void network_change(struct connman_element *element)
 	if (element->state != CONNMAN_ELEMENT_STATE_ERROR)
 		return;
 
-	if (element->error != CONNMAN_ELEMENT_ERROR_DHCP_FAILED)
-		return;
-
 	if (network->connected == FALSE)
 		return;
 
